/*
****************************************
-- XML Indexes
****************************************
*/
USE AdventureWorks2012
GO

CREATE TABLE OrdersXML(
  OrdersId int PRIMARY KEY, 
  OrdersDoc xml NOT NULL)

CREATE PRIMARY XML INDEX ix_orders
 ON OrdersXML(OrdersDoc)

-- Display the columns in the node table (primary XML clustered index)
SELECT
  c.column_id, c.name, t.name AS data_type
 FROM
  sys.columns AS c
  INNER JOIN sys.indexes AS i ON i.object_id = c.object_id
  INNER JOIN sys.types AS t ON t.user_type_id = c.user_type_id
 WHERE
  i.name = 'ix_orders' AND i.type = 1
 ORDER BY
  c.column_id

-- Create secondary structural (path) XML index
CREATE XML INDEX ix_orders_path ON OrdersXML(OrdersDoc)
 USING XML INDEX ix_orders FOR PATH 

-- Create secondary value XML index
CREATE XML INDEX ix_orders_val ON OrdersXML(OrdersDoc)
 USING XML INDEX ix_orders FOR VALUE

-- Create secondary property XML index
CREATE XML INDEX ix_orders_prop ON OrdersXML(OrdersDoc)
 USING XML INDEX ix_orders FOR PROPERTY

-- Cleanup
DROP INDEX ix_orders ON OrdersXML
