/*
****************************************
-- FOR XML
****************************************
*/
USE AdventureWorks2012
GO

-- FOR XML RAW
SELECT TOP 10
  Customer.CustomerID, OrderHeader.SalesOrderID, OrderHeader.OrderDate
 FROM
  Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
   ON OrderHeader.CustomerID = Customer.CustomerID
 ORDER BY
  Customer.CustomerID
 FOR XML RAW 

-- FOR XML AUTO
SELECT TOP 10
  Customer.CustomerID, OrderHeader.SalesOrderID, OrderHeader.OrderDate
 FROM
  Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
   ON OrderHeader.CustomerID = Customer.CustomerID
 ORDER BY
  Customer.CustomerID
 FOR XML AUTO

-- Change hierarchy to Orders > Customer
SELECT TOP 10 
  OrderHeader.SalesOrderID, OrderHeader.OrderDate, Customer.CustomerID
 FROM
  Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
   ON OrderHeader.CustomerID = Customer.CustomerID
 ORDER BY
  Customer.CustomerID
 FOR XML AUTO

-- FOR XML EXPLICIT
SELECT
  1 AS Tag, -- Tag this resultset as level 1
  NULL AS Parent,  -- Level 1 has no parent
  CustomerID AS [Customer!1!CustomerID], -- level 1 value
  NULL AS [SalesOrder!2!SalesOrderID],  -- level 2 value
  NULL AS [SalesOrder!2!OrderDate]  -- level 2 value
 FROM Sales.Customer AS Customer
 WHERE Customer.CustomerID IN(11077, 11078)
 UNION ALL
 SELECT
  2, -- Tag this resultset as level 2
  1, -- Link to parent at level 1
  Customer.CustomerID, 
  OrderHeader.SalesOrderID, 
  OrderHeader.OrderDate
 FROM Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
	 ON OrderHeader.CustomerID = Customer.CustomerID
 WHERE Customer.CustomerID IN(11077, 11078)
 ORDER BY
  [Customer!1!CustomerID], [SalesOrder!2!SalesOrderID]
 FOR XML EXPLICIT

-- FOR XML EXPLICIT with !ELEMENT
SELECT
  1 AS Tag, -- Tag this resultset as level 1
  NULL AS Parent,  -- Level 1 has no parent
  CustomerID AS [Customer!1!CustomerID], -- level 1 value
  NULL AS [SalesOrder!2!SalesOrderID],  -- level 2 value
  NULL AS [SalesOrder!2!OrderDate!ELEMENT] -- level 2 value rendered as an element
 FROM Sales.Customer AS Customer
 WHERE Customer.CustomerID IN(11077, 11078)
 UNION ALL
 SELECT
  2, -- Tag this resultset as level 2
  1, -- Link to parent at level 1
  Customer.CustomerID, 
  OrderHeader.SalesOrderID, 
  OrderHeader.OrderDate
 FROM Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
	 ON OrderHeader.CustomerID = Customer.CustomerID
 WHERE Customer.CustomerID IN(11077, 11078)
 ORDER BY
  [Customer!1!CustomerID], [SalesOrder!2!SalesOrderID]
 FOR XML EXPLICIT

-- FOR XML EXPLICIT with 3 levels
SELECT
  1 AS Tag,
  NULL AS Parent,
  CustomerID AS [Customer!1!CustomerID], 
  NULL AS [SalesOrder!2!SalesOrderID], 
  NULL AS [SalesOrder!2!TotalDue],
  NULL AS [SalesOrder!2!OrderDate!ELEMENT],
  NULL AS [SalesOrder!2!ShipDate!ELEMENT],
  NULL AS [SalesDetail!3!ProductID],
  NULL AS [SalesDetail!3!OrderQty],
  NULL AS [SalesDetail!3!LineTotal]
 FROM Sales.Customer AS Customer
 WHERE Customer.CustomerID IN(11077, 11078)
 UNION ALL
 SELECT 
  2,
  1,
  Customer.CustomerID, 
  OrderHeader.SalesOrderID, 
  OrderHeader.TotalDue,
  OrderHeader.OrderDate,
  OrderHeader.ShipDate,
  NULL,
  NULL,
  NULL
 FROM Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
   ON OrderHeader.CustomerID = Customer.CustomerID
 WHERE Customer.CustomerID IN(11077, 11078)
 UNION ALL
 SELECT 
  3,
  2,
  Customer.CustomerID, 
  OrderHeader.SalesOrderID, 
  OrderHeader.TotalDue,
  OrderHeader.OrderDate,
  OrderHeader.ShipDate,
  OrderDetail.ProductID,
  OrderDetail.OrderQty,
  OrderDetail.LineTotal
 FROM Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader AS OrderHeader
   ON OrderHeader.CustomerID = Customer.CustomerID
  INNER JOIN Sales.SalesOrderDetail AS OrderDetail
   ON OrderDetail.SalesOrderID = OrderHeader.SalesOrderID
 WHERE Customer.CustomerID IN(11077, 11078)
 ORDER BY [Customer!1!CustomerID], [SalesOrder!2!SalesOrderID]
 FOR XML EXPLICIT

-- FOR XML AUTO with TYPE
SELECT 
  CustomerID,
  (SELECT SalesOrderID, TotalDue, OrderDate, ShipDate
   FROM Sales.SalesOrderHeader AS OrderHeader
   WHERE CustomerID = Customer.CustomerID 
   FOR XML AUTO, TYPE) AS OrderHeaders
 FROM
  Sales.Customer AS Customer
 WHERE
  CustomerID IN (11000, 11001)

-- FOR XML PATH - simple example
SELECT
  BusinessEntityID AS [@BusinessEntityID],
  FirstName AS [ContactName/First],
  LastName AS [ContactName/Last],
  EmailAddress AS [ContactEmailAddress/EmailAddress1]
 FROM
  HumanResources.vEmployee
 FOR XML PATH('Contact')

-- FOR XML PATH - 3-level hierarchical example
SELECT 
  CustomerID AS [@CustomerID],
  (SELECT
    SalesOrderID AS [@SalesOrderID], 
    TotalDue AS [@TotalDue], 
    OrderDate, 
    ShipDate,
    (SELECT
      ProductID AS [@ProductID], 
      OrderQty AS [@OrderQty], 
      LineTotal AS [@LineTotal] 
     FROM Sales.SalesOrderDetail
     WHERE SalesOrderID = OrderHeader.SalesOrderID
     FOR XML PATH('OrderDetail'), TYPE) 
   FROM Sales.SalesOrderHeader AS OrderHeader
   WHERE CustomerID = Customer.CustomerID 
   FOR XML PATH('OrderHeader'), TYPE)
 FROM Sales.Customer AS Customer
  INNER JOIN Person.Person AS Contact
   ON Contact.BusinessEntityID = Customer.PersonID
 WHERE CustomerID BETWEEN 11000 AND 11999
 FOR XML PATH ('Customer')

-- FOR XML PATH - using XPath node functions comment() and data()
SELECT 
  Customer.BusinessEntityID AS [@CustomerID],
  Customer.FirstName + ' ' + Customer.LastName AS [comment()],
  (SELECT
    SalesOrderID AS [@SalesOrderID], 
    TotalDue AS [@TotalDue], 
    OrderDate, 
    ShipDate,
    (SELECT ProductID AS [data()]
     FROM Sales.SalesOrderDetail
     WHERE SalesOrderID = OrderHeader.SalesOrderID
     FOR XML PATH('')) AS [ProductIDs] 
   FROM Sales.SalesOrderHeader AS OrderHeader
   WHERE CustomerID = Customer.BusinessEntityID 
   FOR XML PATH('OrderHeader'), TYPE)
 FROM Sales.vIndividualCustomer AS Customer
 WHERE BusinessEntityID IN (11000, 11001)
 FOR XML PATH ('Customer')

-- FOR XML ROOT
SELECT
  Customer.CustomerID, 
  OrderDetail.SalesOrderID,
  OrderDetail.OrderDate
 FROM
  Sales.Customer AS Customer
   INNER JOIN Sales.SalesOrderHeader	AS OrderDetail
    ON OrderDetail.CustomerID = Customer.CustomerID
 WHERE
  Customer.CustomerID IN (11000, 11001)
 ORDER BY
  Customer.CustomerID
 FOR XML AUTO, ROOT('Orders')

-- FOR XML XMLSCHEMA
SELECT
  Customer.CustomerID, 
  OrderDetail.SalesOrderID,
  OrderDetail.OrderDate
 FROM
  Sales.Customer AS Customer
   INNER JOIN Sales.SalesOrderHeader	AS OrderDetail
    ON OrderDetail.CustomerID = Customer.CustomerID
 WHERE
  Customer.CustomerID IN (11000, 11001)
 ORDER BY
  Customer.CustomerID
 FOR XML AUTO, ROOT('Orders'), XMLSCHEMA

-- FOR XML AUTO ELEMENTS
SELECT
	Customer.CustomerID, 
	OrderDetail.SalesOrderID,
	OrderDetail.OrderDate
 FROM
	Sales.Customer AS Customer
  INNER JOIN Sales.SalesOrderHeader	AS OrderDetail
   ON OrderDetail.CustomerID = Customer.CustomerID
 WHERE
	Customer.CustomerID IN (11000, 11001)
 ORDER BY
	Customer.CustomerID
 FOR XML AUTO, ROOT('Orders'), ELEMENTS
