﻿using System;
using System.Linq;
using System.Windows.Forms;

using DemoRestClientTest.CustomerDataService;

namespace DemoRestClientTest
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

    // - The "." after "localhost" is a hack to enable Fiddler packet-sniffing
    // - Change the port number accordingly for your dev machine
    public const string ServiceUri =
     "http://localhost.:1055/CustomerDataService.svc";

    private void btnTestClient_Click(object sender, EventArgs e)
    {
      // Create the client context
      var ctx = new SampleDbEntities(new Uri(ServiceUri));

      // Construct an OData URI query
      var q =
        from cust in ctx.Customers.Expand("OrderHeaders")
        where !cust.FirstName.StartsWith("A")
        orderby cust.LastName descending
        select cust;

      // Call the service (issues an HTTP GET)
      var customers = q.ToList();

      if (customers.Count == 0)
      {
        MessageBox.Show("There are no matching customers");
        return;
      }

      var firstCustomer = customers[0];
      var orderCount = firstCustomer.OrderHeaders.Count;
      MessageBox.Show(string.Format(
        "Customer {0} has {1} order(s)", firstCustomer.CustomerId, orderCount));

      // Change the customer's name
      firstCustomer.FirstName = "Cassie";
      firstCustomer.LastName = "Hicks";

      // Add a customer order
      var newOrder = new OrderHeader()
      {
        ShipVia = "Regular Mail",
        OrderStatus = "Open",
      };
      firstCustomer.OrderHeaders.Add(newOrder);

      ctx.UpdateObject(firstCustomer);
      ctx.AddRelatedObject(firstCustomer, "OrderHeaders", newOrder);

      // Issues an HTTP MERGE to update the customer's name, 
      //  and an HTTP POST to INSERT the customer's new order
      ctx.SaveChanges();
    }

    private void btnCustomServiceOperation_Click(object sender, EventArgs e)
    {
      var ctx = new SampleDbEntities(new Uri(ServiceUri));

      var list = ctx.GetCustomersByBalance(15).ToList();

      MessageBox.Show(string.Format("Customers with balance greater than 15: {0}", list.Count));
    }

  }
}

namespace DemoRestClientTest.CustomerDataService
{
  using System.Data.Services.Client;
  using System.Collections.Generic;

  public partial class SampleDbEntities
  {
    public DataServiceQuery<Customer> GetCustomersByBalance(decimal minBalance)
    {
      return base
        .CreateQuery<Customer>("GetCustomersByBalance")
        .AddQueryOption("minBalance", string.Format("{0}M", minBalance));
    }

  }
}
