﻿using System;
using System.Data.Services;
using System.Data.Services.Common;
using System.Linq;
using System.Linq.Expressions;
using System.ServiceModel.Web;

namespace DemoRestService
{
  public class CustomerDataService : DataService<SampleDbEntities>
  {
    public static void InitializeService(DataServiceConfiguration config)
    {
      config.SetEntitySetAccessRule("Customers", EntitySetRights.All);
      config.SetEntitySetAccessRule("OrderHeaders", EntitySetRights.All);
      config.SetEntitySetAccessRule("Employees", EntitySetRights.AllRead);
      
      config.SetServiceOperationAccessRule("GetCustomersByBalance", ServiceOperationRights.All); 
      
      config.DataServiceBehavior.MaxProtocolVersion = DataServiceProtocolVersion.V2;
    }

    [WebGet]
    public IQueryable<Customer> GetCustomersByBalance(decimal minBalance)
    {
      return
        from customer in base.CurrentDataSource.Customers
        where customer.Balance > minBalance
        select customer;
    }

    protected override void OnStartProcessingRequest(ProcessRequestArgs args)
    {
      base.OnStartProcessingRequest(args);

      // add code here...
    }

    protected override void HandleException(HandleExceptionArgs args)
    {
      base.HandleException(args);

      // add code here...
    }

#if IMPLEMENT_INTERCEPTORS
    [QueryInterceptor("Customers")]
    public Expression<Func<Customer, bool>> OnQueryCustomers()
    {
      return c => c.Balance > 15;
    }

    [ChangeInterceptor("Customers")]
    public void OnChangeCustomers(Customer customer, UpdateOperations op)
    {
      if (op == UpdateOperations.Delete)
      {
        throw new DataServiceException("Customers cannot be deleted");
      }
    }
#endif

  }
}
