﻿using System;
using System.Data;
using System.Linq;
using System.ServiceModel.DomainServices.EntityFramework;
using System.ServiceModel.DomainServices.Hosting;
using System.ServiceModel.DomainServices.Server;
using System.Transactions;

namespace DemoSL.Web
{
  [EnableClientAccess()]
  public class SampleRiaService : LinqToEntitiesDomainService<SampleDbEntities>
  {
    public IQueryable<Customer> GetCustomers()
    {
      var q =
        from cust in base.ObjectContext.Customers.Include("OrderHeaders")
        orderby cust.LastName, cust.FirstName
        select cust;

      return q;
    }

    public void InsertCustomer(Customer addedCustomer)
    {
      base.ObjectContext.Customers.AddObject(addedCustomer);
    }

    public void UpdateCustomer(Customer customer)
    {
      var originalCustomer = base.ChangeSet.GetOriginal(customer);

      if (originalCustomer == null)
      {
        base.ObjectContext.Customers.Attach(customer);
      }
      else
      {
        base.ObjectContext.Customers.AttachAsModified(customer, originalCustomer);
      }

      var orderChanges = this.ChangeSet.GetAssociatedChanges
        (customer, o => o.OrderHeaders);

      foreach (OrderHeader order in orderChanges)
      {
        var changeOperation = this.ChangeSet.GetChangeOperation(order);
        switch (changeOperation)
        {
          case ChangeOperation.Insert:
            base.ObjectContext.ObjectStateManager.ChangeObjectState(order, EntityState.Added);
            break;

          case ChangeOperation.Update:
            base.ObjectContext.OrderHeaders.AttachAsModified(order, base.ChangeSet.GetOriginal(order));
            break;

          case ChangeOperation.Delete:
            base.ObjectContext.OrderHeaders.Attach(order);
            base.ObjectContext.DeleteObject(order);
            break;
        }
      }
    }

    public void DeleteCustomer(Customer removedCustomer)
    {
      base.ObjectContext.Customers.Attach(removedCustomer);
      base.ObjectContext.DeleteObject(removedCustomer);

      var orders = this.ChangeSet.GetAssociatedChanges(removedCustomer, o => o.OrderHeaders);
      foreach (OrderHeader order in orders)
      {
        base.ObjectContext.OrderHeaders.Attach(order);
        base.ObjectContext.DeleteObject(order);
      }
    }

    public override bool Submit(ChangeSet changeSet)
    {
      var result = false;
      using (var ts = new TransactionScope())
      {
        var commit = true;

        result = base.Submit(changeSet);
        if (this.ChangeSet.HasError)
        {
          commit = false;
        }

        // Other out-of-band updates run in the same transaction scope

        if (commit)
        {
          ts.Complete();
        }
      }

      return result;
    }

  }
}
