﻿using System;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;

namespace FlixPoll
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();

            DataContext = App.ViewModel;
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        // Handle selection changed on ListBox
        private void MovieListPicker_SelectionChanged
         (object sender, SelectionChangedEventArgs e)
        {
            // If selected index is -1 (no selection) do nothing
            if (movieListPicker.SelectedIndex == -1)
                return;

            // bind the textbox to the movie
            MovieNameTextBox.DataContext =
             (LocalData.MovieTable)movieListPicker.SelectedItem;
        }

        // Load data for the ViewModel Items
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            if (!App.ViewModel.IsDataLoaded)
            {
                App.ViewModel.LoadData();
            }
        }

        private void ApplicationBarIconButton_SaveClick(object sender, EventArgs e)
        {
            // save changes to the movie's name
            App.ViewModel.SaveMovieToLocalDb();
        }

        private void ApplicationBarIconButton_SyncClick(object sender, EventArgs e)
        {
            App.ViewModel.SyncDone +=
             new EventHandler<SyncDoneEventArgs>(Cloud_UpdateDone);
            App.ViewModel.UpdateToCloud();
        }

        private void ApplicationBarIconButton_NewClick(object sender, EventArgs e)
        {
            // save the answers and reset the quiz for the next person
            App.ViewModel.NewQuiz
             ((movieListPicker.SelectedItem as LocalData.MovieTable).MovieId);
        }

        // called after upload to cloud has taken place to refresh local db
        private void Cloud_UpdateDone(object sender, SyncDoneEventArgs e)
        {
            // reset the data stored locally.
            App.ViewModel.DeleteResponsesAfterSync();
            App.ViewModel.ResetMoviesAfterSync();

            // show the message box on the UI thread
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                MessageBox.Show("sync is complete");
            });
        }
    }
}